unit untTopicBrowser;

interface

uses
  Classes, ComCtrls, Controls,
  untTopic, SysUtils, untGlobal, Forms,
  untTool, ExtCtrls, windows, untBBSCore, RegExpr, StrUtils,
  untHintWindow, Math, untBBSFramework;

type
  TTopicStateType = (stStatusText, stDownloadState);
  TTopicStateChangeEvent = procedure(Topic: TTopic;
                                     EventType: TTopicStateType) of object;
  TNavigateUrlEvent = procedure(sender : TObject; URL : string) of object;

  TPopupManager = class(TObject)
  private
    class function GetMessages(Msgs : TList; const First, Last : Integer) : string;
    class function ParseMailto(const URI: string): string;
  public
    class procedure ParseJump(const URI: string; var First, Last: Integer);
    class function GetPopupMessages(Topic : TTopic; const URI : string) : string;
  end;

  TTopicBrowser = class(TPanel)
  protected
    FBBSGetTopic: TBBSGetTopic;
    FLoading: Boolean;
    FViewLimit: Integer;
    FReloadTimer: TTimer;
    FNewMsg: Boolean;
    FAutoReload: Boolean;
    FDownloading: Boolean;
    FClearNewMsgFlag: Boolean;
    procedure SetAutoReload(const Value: Boolean);
    procedure AutoReloadTimer(sender : TObject);
    procedure RaiseNavigateUrlEvent(Url : string);
    function  ChangeStatusText(statustext: string): boolean;
  public
    Topic: TTopic;
    Caption: string;
    ImageIndex: integer;
    OnNavigateUrl: TNavigateUrlEvent;
    OnMessageReceived, OnComplete: TNotifyEvent;
    OnChangeDownloadState: TNotifyEvent;
    procedure JumpMessage(msgno : integer); virtual; abstract;
    property    AutoReload : Boolean read FAutoReload write SetAutoReload;
    procedure   SaveScrollPosition(); virtual; abstract;
    procedure   OpenTopic(tp: TTopic); virtual;
    procedure   CloseTopic(); virtual;
    procedure   Reload(); virtual;
    procedure   ClearNewMsg();
    procedure   SearchText(str : string); virtual; abstract;
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   ChangeViewLimit(NewLimit : integer); virtual; abstract;
    procedure Topic_MessageReceived(sender : TObject);  virtual; abstract;
    procedure Topic_ChangeDownloadState(Sender: TObject); virtual;
    procedure Topic_Complete(Sender: TObject); virtual;
  end;

implementation

{ TopicBrowser }

//  \bh 

procedure TTopicBrowser.Topic_Complete(Sender: TObject);
begin
  if Assigned(OnComplete) then
  begin
    OnComplete(self);
  end;
end;

procedure TTopicBrowser.RaiseNavigateUrlEvent(Url: string);
var
  p: Integer;
  urlStr: string;
  msgno: integer;

begin
  if AnsiStartsText('Jump://goto/', Url) then
  begin
    p := Pos('-', Url);
    if p <> 0 then
    begin
      urlStr := Copy(Url, 0, p - 1)
    end else
    begin
      UrlStr := Url;
    end;
    msgno := StrToIntNeo(CopyAfter(UrlStr, 13));
    JumpMessage(msgno);
  end else
  begin
    if Assigned(OnNavigateUrl) then
    begin
      OnNavigateUrl(self, Url);
    end;
  end;
end;
{ --------------------------------------------------------
  pr  : RXgN^
  l  : Ȃ
  ------------------------------------------------------ }
constructor TTopicBrowser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  self.BevelInner := bvNone;
  self.BevelOuter := bvNone;

  FLoading := false;
  ImageIndex := -1;
  FReloadTimer := TTimer.Create(self);
  FReloadTimer.Enabled  := false;
  FReloadTimer.Interval := 60000;
  FReloadTimer.OnTimer  := AutoReloadTimer;
end;

{ --------------------------------------------------------
  pr  : fXgN^
  l  : Ȃ
  ------------------------------------------------------ }
destructor TTopicBrowser.Destroy;
begin

  FReloadTimer.Free;

  inherited;
end;

procedure TTopicBrowser.Reload;
begin
  gBBSCore.DownloadTopic(Topic, self);
end;

{ --------------------------------------------------------
  pr  : gsbNJ
  l  : Ȃ
  ------------------------------------------------------ }
procedure TTopicBrowser.OpenTopic(tp: TTopic);

begin
  if Assigned(Topic) then
  begin
    CloseTopic();
  end;

  Topic := tp;
  if Assigned(Topic) then
  begin
    Topic.IsOpened := true;
    Caption := Topic.Title;
    FNewMsg := false;
    FViewLimit := gConfig.ViewLimit;
    gBBSCore.DownloadTopic(Topic, self);
  end else
  begin
    Caption := '';
  end;
end;

{ --------------------------------------------------------
  ֐: CloseTopic
  pr  : gsbN
    : Ȃ
  ߂l: Ȃ
  l  : Ȃ
  ------------------------------------------------------ }
procedure TTopicBrowser.CloseTopic;
begin
  if Topic <> nil then
  begin
    Topic.IsOpened := false;
    Topic.FreeMessage;
    SaveScrollPosition();
    SetAutoReload(false);
  end;

  Topic := nil;
  Caption  := '';
  ImageIndex := -1;
end;

{ --------------------------------------------------------
  ֐: ClearNewMsg
  pr  : V}[N̎O
    : Ȃ
  ߂l: Ȃ
  l  : Ȃ
  ------------------------------------------------------ }
procedure TTopicBrowser.ClearNewMsg;
begin
  if FClearNewMsgFlag = true then
  begin
    exit;
  end;
  FClearNewMsgFlag := true;
  FNewMsg := false;
  Topic_ChangeDownloadState(Topic);
  FClearNewMsgFlag := false;
end;

//  Cxg 

// ԕω
procedure TTopicBrowser.Topic_ChangeDownloadState(Sender: TObject);
begin
  Case Topic.DownloadState of
    dsNone:
    begin
      if FNewMsg = true then
      begin
        ImageIndex := 0
      end else if FAutoReload = true then
      begin
        ImageIndex := 1
      end else
      begin
        ImageIndex := -1;
      end;

      if FDownloading = true then
      begin
        FDownloading := false;

        // vOANeBułȂ
        // ^Cgo[_ł
        if FNewMsg and (Application.Active = false) then
        begin
          FlashWindow(Application.Handle, True);
        end;
      end;
    end;
    dsError:      ImageIndex := 2;
    dsFreezed:    ImageIndex := 3;
    dsStarting:   ImageIndex := 4;
    dsProcessing: ImageIndex := 5;
    dsDatLoaded:
    begin

      //RestoreScrollPosition;

    end;
  end;

  if Assigned(OnChangeDownloadState) then
  begin
    OnChangeDownloadState(self);
  end;
end;

// I[g[h
procedure TTopicBrowser.AutoReloadTimer(sender: TObject);
begin
  if FNewMsg = false then
  begin
    Reload;
  end;
end;

//  vpeB 

// I[g[hݒ
procedure TTopicBrowser.SetAutoReload(const Value: Boolean);
begin
  FAutoReload := Value;
  FReloadTimer.Enabled := Value;
  Topic_ChangeDownloadState(Topic);
end;

//  vCx[g֐ 

function TTopicBrowser.ChangeStatusText(statustext: string) : boolean;
var
  msgText: string;

begin
  msgText := TPopupManager.GetPopupMessages(Topic, statustext);

  if msgText <> '' then
  begin
    ToolTip.SetHint(msgText);
    Result := True;
  end else
  begin
    ToolTip.UnVisible;
    Result := False;
  end;
end;

{ TPopupManager }

class function TPopupManager.GetMessages(Msgs : TList; const First, Last : Integer) : string;
var
  i: Integer;
  firstPos, lastPos: Integer;
  msg: string;

begin
  // eg. >>0
  if (First < 1) and (Last < 0) then
  begin
    Result := '';
    Exit;
  end;

  firstPos := Max(First - 1, 0);

  lastPos := Min(Last, Msgs.Count);
  Dec(LastPos);

  // eg. >>100000 or >>10-9
  if (firstPos >= msgs.Count) or ((lastPos >= 0) and (firstPos > lastPos)) then
  begin
    Result := '';
    Exit;
  end;

  if lastPos > 0 then
  begin
    for i := firstPos to lastPos do
    begin
      if I > firstPos + 20 then
      begin
        msg := msg + '(ȗ܂)';
        break;
      end else
      begin
        msg := msg + TTopicMessage(Msgs[i]).ForPopup + #10#10
      end;
    end
  end else
  begin
    msg := TTopicMessage(Msgs[FirstPos]).ForPopup;
  end;

  Result := msg;
end;

class function  TPopupManager.ParseMailto(const URI : string): string;
begin
  Result := CopyAfter(URI, 8);
end;

// \n+(-\n+)? only
class procedure  TPopupManager.ParseJump(const URI : string; var First, Last: Integer);
var
  Text : string;
  p : Integer;

begin
  Text := CopyAfter(URI, 13);

  p := Pos('-', Text);
  if p <> 0 then
  begin
    First := StrToInt(Copy(Text, 0, p - 1));
    Last  := StrToInt(Copy(Text, p + 1, Length(Text)));
  end else
  begin
    First := StrToInt(Text);
    Last  := -1;
  end;
end;

class function TPopupManager.GetPopupMessages(Topic: TTopic; const URI: string): string;
var
  first, last: integer;

begin
  if Copy(URI, 1, 7) = 'mailto:' then
  begin
    Result := ParseMailto(URI);
  end else if Copy(URI, 1, 12) = 'jump://goto/' then
  begin
    ParseJump(URI, first, last);
    Result := GetMessages(Topic.MessageList, first, last);
  end else
  begin
    Result := '';
  end;
end;

end.
